@@ -1,5 +1,6 @@
 /*
 
+Copyright 2018 S. Razi Alavizadeh
 Copyright 2014-2015 Adam Reichold
 
 This file is part of qpdfview.
@@ -45,24 +46,17 @@ BookmarkModel* BookmarkModel::fromPath(const QString& path, bool create)
     return model;
 }
 
-QList< QString > BookmarkModel::knownPaths()
+QList< QString > BookmarkModel::paths()
 {
     return s_instances.keys();
 }
 
-void BookmarkModel::forgetPath(const QString& path)
+void BookmarkModel::removePath(const QString& path)
 {
-    QHash< QString, BookmarkModel* >::iterator at = s_instances.find(path);
-
-    if(at != s_instances.end())
-    {
-        delete at.value();
-
-        s_instances.erase(at);
-    }
+    delete s_instances.take(path);
 }
 
-void BookmarkModel::forgetAllPaths()
+void BookmarkModel::removeAllPaths()
 {
     qDeleteAll(s_instances);
     s_instances.clear();
@@ -70,10 +64,10 @@ void BookmarkModel::forgetAllPaths()
 
 void BookmarkModel::addBookmark(const BookmarkItem& bookmark)
 {
-    QList< BookmarkItem >::iterator at = qBinaryFind(m_bookmarks.begin(), m_bookmarks.end(), bookmark.page);
-    int row = at - m_bookmarks.begin();
+    const QVector< BookmarkItem >::iterator at = qLowerBound(m_bookmarks.begin(), m_bookmarks.end(), bookmark.page);
+    const int row = at - m_bookmarks.begin();
 
-    if(at != m_bookmarks.end())
+    if(at != m_bookmarks.end() && at->page == bookmark.page)
     {
         *at = bookmark;
 
@@ -81,9 +75,6 @@ void BookmarkModel::addBookmark(const BookmarkItem& bookmark)
     }
     else
     {
-        at = qUpperBound(m_bookmarks.begin(), m_bookmarks.end(), bookmark.page);
-        row = at - m_bookmarks.begin();
-
         beginInsertRows(QModelIndex(), row, row);
 
         m_bookmarks.insert(at, bookmark);
@@ -94,7 +85,7 @@ void BookmarkModel::addBookmark(const BookmarkItem& bookmark)
 
 void BookmarkModel::removeBookmark(const BookmarkItem& bookmark)
 {
-    const QList< BookmarkItem >::iterator at = qBinaryFind(m_bookmarks.begin(), m_bookmarks.end(), bookmark.page);
+    const QVector< BookmarkItem >::iterator at = qBinaryFind(m_bookmarks.begin(), m_bookmarks.end(), bookmark.page);
     const int row = at - m_bookmarks.begin();
 
     if(at != m_bookmarks.end())
@@ -109,7 +100,7 @@ void BookmarkModel::removeBookmark(const BookmarkItem& bookmark)
 
 void BookmarkModel::findBookmark(BookmarkItem& bookmark) const
 {
-    const QList< BookmarkItem >::const_iterator at = qBinaryFind(m_bookmarks.constBegin(), m_bookmarks.constEnd(), bookmark.page);
+    const QVector< BookmarkItem >::const_iterator at = qBinaryFind(m_bookmarks.constBegin(), m_bookmarks.constEnd(), bookmark.page);
 
     if(at != m_bookmarks.constEnd())
     {
@@ -151,8 +142,9 @@ QVariant BookmarkModel::data(const QModelIndex& index, int role) const
     case Qt::DisplayRole:
         return index.column() == 0 ? bookmark.label : QString::number(bookmark.page);
     case CommentRole:
-    case Qt::ToolTipRole:
         return bookmark.comment;
+    case Qt::ToolTipRole:
+        return QString("<p>%1</p>").arg(bookmark.comment);
     case ModifiedRole:
         return bookmark.modified;
     case Qt::TextAlignmentRole:
