@@ -1,8 +1,9 @@
 /*
 
 Copyright 2014 S. Razi Alavizadeh
-Copyright 2012-2014 Adam Reichold
+Copyright 2012-2014, 2018 Adam Reichold
 Copyright 2014 Dorian Scholz
+Copyright 2018 Egor Zenkov
 
 This file is part of qpdfview.
 
@@ -126,8 +127,8 @@ public:
     RubberBandMode rubberBandMode() const { return m_rubberBandMode; }
     void setRubberBandMode(RubberBandMode rubberBandMode);
 
-    bool searchWasCanceled() const;
-    int searchProgress() const;
+    QSize thumbnailsViewportSize() const { return m_thumbnailsViewportSize; }
+    void setThumbnailsViewportSize(QSize thumbnailsViewportSize);
 
     Qt::Orientation thumbnailsOrientation() const { return m_thumbnailsOrientation; }
     void setThumbnailsOrientation(Qt::Orientation thumbnailsOrientation);
@@ -135,10 +136,16 @@ public:
     const QVector< ThumbnailItem* >& thumbnailItems() const { return m_thumbnailItems; }
     QGraphicsScene* thumbnailsScene() const { return m_thumbnailsScene; }
 
-    QStandardItemModel* outlineModel() const { return m_outlineModel; }
-    QStandardItemModel* propertiesModel() const { return m_propertiesModel; }
+    QAbstractItemModel* outlineModel() const { return m_outlineModel.data(); }
+    QAbstractItemModel* propertiesModel() const { return m_propertiesModel.data(); }
 
-    QStandardItemModel* fontsModel() const;
+    QSet< QByteArray > saveExpandedPaths() const;
+    void restoreExpandedPaths(const QSet< QByteArray >& expandedPaths);
+
+    QAbstractItemModel* fontsModel() const;
+
+    bool searchWasCanceled() const;
+    int searchProgress() const;
 
     QString searchText() const;
     bool searchMatchCase() const;
@@ -146,6 +153,24 @@ public:
 
     QPair< QString, QString > searchContext(int page, const QRectF& rect) const;
 
+    bool hasSearchResults();
+
+    QString resolveFileName(QString fileName) const;
+    QUrl resolveUrl(QUrl url) const;
+
+    struct SourceLink
+    {
+        QString name;
+        int line;
+        int column;
+
+        operator bool() const { return !name.isNull(); }
+
+    };
+
+    SourceLink sourceLink(QPoint pos);
+    void openInSourceEditor(const SourceLink& sourceLink);
+
 signals:
     void documentChanged();
     void documentModified();
@@ -240,8 +265,8 @@ protected slots:
 
     void on_pages_rubberBandFinished();
 
-    void on_pages_editSourceRequested(int page, const QPointF& pos);
-    void on_pages_zoomToSelectionRequested(int page, const QRectF& rect);
+    void on_pages_zoomToSelection(int page, const QRectF& rect);
+    void on_pages_openInSourceEditor(int page, QPointF pos);
 
     void on_pages_wasModified();
 
@@ -315,20 +340,23 @@ private:
 
     QGraphicsRectItem* m_highlight;
 
+    QSize m_thumbnailsViewportSize;
     Qt::Orientation m_thumbnailsOrientation;
+
     QGraphicsScene* m_thumbnailsScene;
 
-    QStandardItemModel* m_outlineModel;
-    QStandardItemModel* m_propertiesModel;
+    QScopedPointer< QAbstractItemModel > m_outlineModel;
+    QScopedPointer< QAbstractItemModel > m_propertiesModel;
 
     bool checkDocument(const QString& filePath, Model::Document* document, QVector< Model::Page* >& pages);
 
-    void loadFallbackOutline();
     void loadDocumentDefaults();
 
     void adjustScrollBarPolicy();
-    void disconnectVerticalScrollBar();
-    void reconnectVerticalScrollBar();
+
+    bool m_verticalScrollBarChangedBlocked;
+
+    class VerticalScrollBarChangedBlocker;
 
     void prepareDocument(Model::Document* document, const QVector< Model::Page* >& pages);
     void preparePages();
@@ -357,4 +385,6 @@ private:
 
 } // qpdfview
 
+Q_DECLARE_METATYPE(qpdfview::DocumentView::SourceLink)
+
 #endif // DOCUMENTVIEW_H
