@@ -1,7 +1,8 @@
 /*
 
+Copyright 2018 Marshall Banana
 Copyright 2013 Benjamin Eltzner
-Copyright 2013 Adam Reichold
+Copyright 2013, 2018 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -30,21 +31,25 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QTextBrowser>
 #include <QVBoxLayout>
 
+#include "settings.h"
+
 namespace qpdfview
 {
 
 HelpDialog::HelpDialog(QWidget* parent) : QDialog(parent)
 {
+    setWindowTitle(tr("Help") + QLatin1String(" - qpdfview"));
+
     m_textBrowser = new QTextBrowser(this);
     m_textBrowser->setTextInteractionFlags(Qt::TextBrowserInteraction | Qt::TextSelectableByKeyboard);
-    m_textBrowser->setSearchPaths(QStringList() << QDir(QApplication::applicationDirPath()).filePath("data") << DATA_INSTALL_PATH);
+    m_textBrowser->setSearchPaths(QStringList() << QDir(QApplication::applicationDirPath()).filePath("data") << DATA_INSTALL_PATH << ":/");
 
     //: Please replace by file name of localized help if available, e.g. "help_fr.html".
-    m_textBrowser->setSource(QUrl(tr("help.html")));
+    m_textBrowser->setSource(QUrl::fromLocalFile(tr("help.html")));
 
     if(m_textBrowser->document()->isEmpty())
     {
-        m_textBrowser->setSource(QUrl("help.html"));
+        m_textBrowser->setSource(QUrl::fromLocalFile("help.html"));
     }
 
     m_dialogButtonBox = new QDialogButtonBox(QDialogButtonBox::Close, Qt::Horizontal, this);
@@ -80,9 +85,16 @@ HelpDialog::HelpDialog(QWidget* parent) : QDialog(parent)
     layout()->addWidget(m_searchLineEdit);
     layout()->addWidget(m_dialogButtonBox);
 
+    resize(Settings::instance()->mainWindow().contentsDialogSize(sizeHint()));
+
     m_searchLineEdit->setFocus();
 }
 
+HelpDialog::~HelpDialog()
+{
+    Settings::instance()->mainWindow().setContentsDialogSize(size());
+}
+
 void HelpDialog::on_findPrevious_triggered()
 {
     if(!m_searchLineEdit->text().isEmpty() && m_textBrowser->find(m_searchLineEdit->text(), QTextDocument::FindBackward))
