@@ -1,6 +1,7 @@
 /*
 
-Copyright 2012-2013 Adam Reichold
+Copyright 2018 Marshall Banana
+Copyright 2012-2013, 2018 Adam Reichold
 Copyright 2014 Dorian Scholz
 Copyright 2012 Michał Trybus
 Copyright 2013 Chris Young
@@ -44,6 +45,16 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <synctex_parser.h>
 
+#ifndef HAS_SYNCTEX_2
+
+typedef synctex_scanner_t synctex_scanner_p;
+typedef synctex_node_t synctex_node_p;
+
+#define synctex_scanner_next_result(scanner) synctex_next_result(scanner)
+#define synctex_display_query(scanner, file, line, column, page) synctex_display_query(scanner, file, line, column)
+
+#endif // HAS_SYNCTEX_2
+
 #endif // WITH_SYNCTEX
 
 #include "documentview.h"
@@ -103,26 +114,35 @@ QList< File > files;
 
 MainWindow* mainWindow = 0;
 
-void loadTranslators()
+bool loadTranslator(QTranslator* const translator, const QString& fileName, const QString& path)
 {
-    QTranslator* toolkitTranslator = new QTranslator(qApp);
-    QTranslator* applicationTranslator = new QTranslator(qApp);
-
 #if QT_VERSION >= QT_VERSION_CHECK(4,8,0)
 
-    if(toolkitTranslator->load(QLocale::system(), "qt", "_", QLibraryInfo::location(QLibraryInfo::TranslationsPath))) { qApp->installTranslator(toolkitTranslator); }
-
-    if(applicationTranslator->load(QLocale::system(), "qpdfview", "_", QDir(QApplication::applicationDirPath()).filePath("data"))) { qApp->installTranslator(applicationTranslator); }
-    else if(applicationTranslator->load(QLocale::system(), "qpdfview", "_", DATA_INSTALL_PATH)) { qApp->installTranslator(applicationTranslator); }
+    const bool ok = translator->load(QLocale::system(), fileName, "_", path);
 
 #else
 
-    if(toolkitTranslator->load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath))) { qApp->installTranslator(toolkitTranslator); }
-
-    if(applicationTranslator->load("qpdfview_" + QLocale::system().name(), QDir(QApplication::applicationDirPath()).filePath("data"))) { qApp->installTranslator(applicationTranslator); }
-    else if(applicationTranslator->load("qpdfview_" + QLocale::system().name(), DATA_INSTALL_PATH)) { qApp->installTranslator(applicationTranslator); }
+    const bool ok = translator->load(fileName + "_" + QLocale::system().name(), path);
 
 #endif // QT_VERSION
+
+    if(ok)
+    {
+        qApp->installTranslator(translator);
+    }
+
+    return ok;
+}
+
+void loadTranslators()
+{
+    QTranslator* toolkitTranslator = new QTranslator(qApp);
+    loadTranslator(toolkitTranslator, "qt", QLibraryInfo::location(QLibraryInfo::TranslationsPath));
+
+    QTranslator* applicationTranslator = new QTranslator(qApp);
+    if(loadTranslator(applicationTranslator, "qpdfview", QDir(QApplication::applicationDirPath()).filePath("data"))) {}
+    else if(loadTranslator(applicationTranslator, "qpdfview", DATA_INSTALL_PATH)) {}
+    else if(loadTranslator(applicationTranslator, "qpdfview", ":/")) {}
 }
 
 void parseCommandLineArguments()
@@ -187,7 +207,7 @@ void parseCommandLineArguments()
             else if(argument == QLatin1String("--choose-instance"))
             {
                 bool ok = false;
-                const QString chosenInstanceName = QInputDialog::getItem(0, QObject::tr("Choose instance"), QObject::tr("Instance:"), Database::instance()->loadInstanceNames(), 0, true, &ok);
+                const QString chosenInstanceName = QInputDialog::getItem(0, MainWindow::tr("Choose instance"), MainWindow::tr("Instance:"), Database::instance()->knownInstanceNames(), 0, true, &ok);
 
                 if(ok)
                 {
@@ -316,13 +336,11 @@ void resolveSourceReferences()
 
         if(!file.sourceName.isNull())
         {
-            synctex_scanner_t scanner = synctex_scanner_new_with_output_file(file.filePath.toLocal8Bit(), 0, 1);
-
-            if(scanner != 0)
+            if(synctex_scanner_p scanner = synctex_scanner_new_with_output_file(file.filePath.toLocal8Bit(), 0, 1))
             {
-                if(synctex_display_query(scanner, file.sourceName.toLocal8Bit(), file.sourceLine, file.sourceColumn) > 0)
+                if(synctex_display_query(scanner, file.sourceName.toLocal8Bit(), file.sourceLine, file.sourceColumn, -1) > 0)
                 {
-                    for(synctex_node_t node = synctex_next_result(scanner); node != 0; node = synctex_next_result(scanner))
+                    for(synctex_node_p node = synctex_scanner_next_result(scanner); node != 0; node = synctex_scanner_next_result(scanner))
                     {
                         int page = synctex_node_page(node);
                         QRectF enclosingBox(synctex_node_box_visible_h(node), synctex_node_box_visible_v(node), synctex_node_box_visible_width(node), synctex_node_box_visible_height(node));
@@ -343,7 +361,7 @@ void resolveSourceReferences()
             }
             else
             {
-                qWarning() << QObject::tr("SyncTeX data for '%1' could not be found.").arg(file.filePath);
+                qWarning() << DocumentView::tr("SyncTeX data for '%1' could not be found.").arg(file.filePath);
             }
         }
     }
@@ -359,15 +377,7 @@ void activateUniqueInstance()
 
     if(unique)
     {
-        QString serviceName = QApplication::organizationDomain();
-
-        if(!instanceName.isEmpty())
-        {
-            serviceName.append('.');
-            serviceName.append(instanceName);
-        }
-
-        QScopedPointer< QDBusInterface > interface(new QDBusInterface(serviceName, "/MainWindow", "local.qpdfview.MainWindow", QDBusConnection::sessionBus()));
+        QScopedPointer< QDBusInterface > interface(MainWindowAdaptor::createInterface());
 
         if(interface->isValid())
         {
@@ -385,6 +395,11 @@ void activateUniqueInstance()
                 }
             }
 
+            if(!files.isEmpty())
+            {
+                interface->call("saveDatabase");
+            }
+
             if(!searchText.isEmpty())
             {
                 interface->call("startSearch", searchText);
@@ -396,17 +411,7 @@ void activateUniqueInstance()
         {
             mainWindow = new MainWindow();
 
-            new MainWindowAdaptor(mainWindow);
-
-            if(!QDBusConnection::sessionBus().registerService(serviceName))
-            {
-                qCritical() << QDBusConnection::sessionBus().lastError().message();
-
-                delete mainWindow;
-                exit(ExitDBusError);
-            }
-
-            if(!QDBusConnection::sessionBus().registerObject("/MainWindow", mainWindow))
+            if(MainWindowAdaptor::createAdaptor(mainWindow) == 0)
             {
                 qCritical() << QDBusConnection::sessionBus().lastError().message();
 
@@ -414,8 +419,6 @@ void activateUniqueInstance()
                 exit(ExitDBusError);
             }
         }
-
-        return;
     }
     else
     {
@@ -466,7 +469,13 @@ int main(int argc, char** argv)
 
     QApplication::setApplicationVersion(APPLICATION_VERSION);
 
-    QApplication::setWindowIcon(QIcon(":icons/qpdfview.svg"));
+    QApplication::setWindowIcon(QIcon(":icons/qpdfview"));
+
+#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+
+    QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+
+#endif // QT_VERSION
 
     loadTranslators();
 
@@ -486,6 +495,11 @@ int main(int argc, char** argv)
         mainWindow->jumpToPageOrOpenInNewTab(file.filePath, file.page, true, file.enclosingBox, quiet);
     }
 
+    if(!files.isEmpty())
+    {
+        mainWindow->saveDatabase();
+    }
+
     if(!searchText.isEmpty())
     {
         mainWindow->startSearch(searchText);
