@@ -1,7 +1,8 @@
 /*
 
 Copyright 2014 S. Razi Alavizadeh
-Copyright 2012-2015 Adam Reichold
+Copyright 2012-2018 Adam Reichold
+Copyright 2018 Pavel Sanda
 Copyright 2014 Dorian Scholz
 
 This file is part of qpdfview.
@@ -30,6 +31,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #include <QLabel>
 #include <QMenu>
 #include <QMouseEvent>
+#include <QProcess>
 #include <QScrollBar>
 #include <QTimer>
 #include <QToolTip>
@@ -37,14 +39,15 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "searchmodel.h"
 
+namespace qpdfview
+{
+
 namespace
 {
 
-using namespace qpdfview;
-
 inline bool isPrintable(const QString& string)
 {
-    foreach(QChar character, string)
+    foreach(const QChar& character, string)
     {
         if(!character.isPrint())
         {
@@ -62,9 +65,6 @@ inline QModelIndex firstIndex(const QModelIndexList& indexes)
 
 } // anonymous
 
-namespace qpdfview
-{
-
 GraphicsCompositionModeEffect::GraphicsCompositionModeEffect(QPainter::CompositionMode compositionMode, QObject* parent) : QGraphicsEffect(parent),
     m_compositionMode(compositionMode)
 {
@@ -106,7 +106,31 @@ int ProxyStyle::styleHint(StyleHint hint, const QStyleOption* option, const QWid
     return QProxyStyle::styleHint(hint, option, widget, returnData);
 }
 
-SearchableMenu::SearchableMenu(const QString& title, QWidget* parent) : QMenu(title, parent),
+ToolTipMenu::ToolTipMenu(QWidget* parent) : QMenu(parent)
+{
+}
+
+ToolTipMenu::ToolTipMenu(const QString& title, QWidget* parent) : QMenu(title, parent)
+{
+}
+
+bool ToolTipMenu::event(QEvent* event)
+{
+    const QAction* const action = activeAction();
+
+    if(event->type() == QEvent::ToolTip && action != 0 && !action->data().isNull())
+    {
+        QToolTip::showText(static_cast< QHelpEvent* >(event)->globalPos(), action->toolTip());
+    }
+    else
+    {
+        QToolTip::hideText();
+    }
+
+    return QMenu::event(event);
+}
+
+SearchableMenu::SearchableMenu(const QString& title, QWidget* parent) : ToolTipMenu(title, parent),
     m_searchable(false),
     m_text()
 {
@@ -161,27 +185,32 @@ void SearchableMenu::keyPressEvent(QKeyEvent* event)
         return;
     }
 
-    setActiveAction(0);
+    QAction* firstVisibleAction = 0;
 
     foreach(QAction* action, actions())
     {
-        if(!action->data().isNull()) // Modify only flagged actions
+        if(action->data().isNull()) // Modify only flagged actions
         {
-            const bool visible = action->text().contains(m_text, Qt::CaseInsensitive);
+            continue;
+        }
 
-            action->setVisible(visible);
+        const bool visible = action->text().contains(m_text, Qt::CaseInsensitive);
 
-            if(visible && activeAction() == 0)
-            {
-                setActiveAction(action);
-            }
+        action->setVisible(visible);
+
+        if(visible && firstVisibleAction == 0)
+        {
+            firstVisibleAction = action;
         }
     }
 
+    setActiveAction(firstVisibleAction);
+
     QToolTip::showText(mapToGlobal(rect().topLeft()), tr("Search for '%1'...").arg(m_text), this);
 }
 
-TabBar::TabBar(QWidget* parent) : QTabBar(parent)
+TabBar::TabBar(QWidget* parent) : QTabBar(parent),
+    m_dragIndex(-1)
 {
 }
 
@@ -212,8 +241,6 @@ QSize TabBar::tabSizeHint(int index) const
 
 void TabBar::mousePressEvent(QMouseEvent* event)
 {
-    QTabBar::mousePressEvent(event);
-
     if(event->button() == Qt::MidButton)
     {
         const int index = tabAt(event->pos());
@@ -221,19 +248,75 @@ void TabBar::mousePressEvent(QMouseEvent* event)
         if(index != -1)
         {
             emit tabCloseRequested(index);
+
+            event->accept();
+            return;
         }
     }
+    else if(event->modifiers() == Qt::ShiftModifier && event->button() == Qt::LeftButton)
+    {
+        const int index = tabAt(event->pos());
+
+        if(index != -1)
+        {
+            m_dragIndex = index;
+            m_dragPos = event->pos();
+
+            event->accept();
+            return;
+        }
+    }
+
+    QTabBar::mousePressEvent(event);
+}
+
+void TabBar::mouseMoveEvent(QMouseEvent* event)
+{
+    QTabBar::mouseMoveEvent(event);
+
+    if(m_dragIndex != -1)
+    {
+        if((event->pos() - m_dragPos).manhattanLength() >= QApplication::startDragDistance())
+        {
+            emit tabDragRequested(m_dragIndex);
+
+            m_dragIndex = -1;
+        }
+    }
+}
+
+void TabBar::mouseReleaseEvent(QMouseEvent* event)
+{
+    QTabBar::mouseReleaseEvent(event);
+
+    m_dragIndex = -1;
 }
 
 TabWidget::TabWidget(QWidget* parent) : QTabWidget(parent),
     m_tabBarPolicy(TabBarAsNeeded),
     m_spreadTabs(false)
 {
-    setTabBar(new TabBar(this));
+    TabBar* tabBar = new TabBar(this);
 
-    tabBar()->setContextMenuPolicy(Qt::CustomContextMenu);
+    tabBar->setContextMenuPolicy(Qt::CustomContextMenu);
 
-    connect(tabBar(), SIGNAL(customContextMenuRequested(QPoint)), SLOT(on_tabBar_customContextMenuRequested(QPoint)));
+    connect(tabBar, SIGNAL(tabDragRequested(int)), SIGNAL(tabDragRequested(int)));
+    connect(tabBar, SIGNAL(customContextMenuRequested(QPoint)), SLOT(on_tabBar_customContextMenuRequested(QPoint)));
+
+    setTabBar(tabBar);
+}
+
+int TabWidget::addTab(QWidget* const widget, const bool nextToCurrent,
+                      const QString& label, const QString& toolTip)
+{
+    const int index = nextToCurrent
+            ? insertTab(currentIndex() + 1, widget, label)
+            : QTabWidget::addTab(widget, label);
+
+    setTabToolTip(index, toolTip);
+    setCurrentIndex(index);
+
+    return index;
 }
 
 TabWidget::TabBarPolicy TabWidget::tabBarPolicy() const
@@ -275,9 +358,38 @@ void TabWidget::setSpreadTabs(bool spreadTabs)
     }
 }
 
-void TabWidget::on_tabBar_customContextMenuRequested(const QPoint& pos)
+void TabWidget::previousTab()
 {
-    emit tabContextMenuRequested(tabBar()->mapToGlobal(pos), tabBar()->tabAt(pos));
+    int index = currentIndex() - 1;
+
+    if(index < 0)
+    {
+        index = count() - 1;
+    }
+
+    setCurrentIndex(index);
+}
+
+void TabWidget::nextTab()
+{
+    int index = currentIndex() + 1;
+
+    if(index >= count())
+    {
+        index = 0;
+    }
+
+    setCurrentIndex(index);
+}
+
+void TabWidget::on_tabBar_customContextMenuRequested(QPoint pos)
+{
+    const int index = tabBar()->tabAt(pos);
+
+    if(index != -1)
+    {
+        emit tabContextMenuRequested(tabBar()->mapToGlobal(pos), tabBar()->tabAt(pos));
+    }
 }
 
 void TabWidget::tabInserted(int index)
@@ -660,14 +772,14 @@ QString MappingSpinBox::textFromValue(int val) const
 int MappingSpinBox::valueFromText(const QString& text) const
 {
     bool ok = false;
-    int val = m_mapper->valueFromText(text, ok);
+    int value = m_mapper->valueFromText(text, ok);
 
     if(!ok)
     {
-        val = SpinBox::valueFromText(text);
+        value = SpinBox::valueFromText(text);
     }
 
-    return val;
+    return value;
 }
 
 QValidator::State MappingSpinBox::validate(QString& input, int& pos) const
@@ -691,6 +803,7 @@ int getMappedNumber(MappingSpinBox::TextValueMapper* mapper,
     MappingSpinBox* mappingSpinBox = new MappingSpinBox(mapper, dialog);
     mappingSpinBox->setRange(min, max);
     mappingSpinBox->setValue(value);
+    mappingSpinBox->selectAll();
 
     QDialogButtonBox* dialogButtonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, dialog);
     QObject::connect(dialogButtonBox, SIGNAL(accepted()), dialog, SLOT(accept()));
@@ -791,11 +904,88 @@ void SearchLineEdit::on_timeout()
     emit searchInitiated(text());
 }
 
-void SearchLineEdit::on_returnPressed(const Qt::KeyboardModifiers& modifiers)
+void SearchLineEdit::on_returnPressed(Qt::KeyboardModifiers modifiers)
 {
     stopTimer();
 
     emit searchInitiated(text(), modifiers == Qt::ShiftModifier);
 }
 
+Splitter::Splitter(Qt::Orientation orientation, QWidget* parent) : QSplitter(orientation, parent),
+    m_currentIndex(0)
+{
+    connect(qApp, SIGNAL(focusChanged(QWidget*,QWidget*)), this, SLOT(on_focusChanged(QWidget*,QWidget*)));
+}
+
+QWidget* Splitter::currentWidget() const
+{
+    return widget(m_currentIndex);
+}
+
+void Splitter::setCurrentWidget(QWidget* const currentWidget)
+{
+    for(int index = 0, count = this->count(); index < count; ++index)
+    {
+        QWidget* const widget = this->widget(index);
+
+        if(currentWidget == widget)
+        {
+            if(m_currentIndex != index)
+            {
+                m_currentIndex = index;
+
+                emit currentWidgetChanged(currentWidget);
+            }
+
+            return;
+        }
+    }
+}
+
+void Splitter::setUniformSizes()
+{
+    int size;
+
+    switch(orientation())
+    {
+    default:
+    case Qt::Horizontal:
+        size = width();
+        break;
+    case Qt::Vertical:
+        size = height();
+        break;
+    }
+
+    QList< int > sizes;
+
+    for(int index = 0, count = this->count(); index < count; ++index)
+    {
+        sizes.append(size / count);
+    }
+
+    setSizes(sizes);
+}
+
+void Splitter::on_focusChanged(QWidget* /* old */, QWidget* now)
+{
+    for(QWidget* currentWidget = now; currentWidget != 0; currentWidget = currentWidget->parentWidget())
+    {
+        if(currentWidget->parentWidget() == this)
+        {
+            setCurrentWidget(currentWidget);
+
+            return;
+        }
+    }
+}
+
+void openInNewWindow(const QString& filePath, int page)
+{
+    QProcess::startDetached(
+        QApplication::applicationFilePath(),
+        QStringList() << QString("%2#%1").arg(page).arg(filePath)
+    );
+}
+
 } // qpdfview
