@@ -1,7 +1,7 @@
 /*
 
 Copyright 2015 S. Razi Alavizadeh
-Copyright 2012-2015 Adam Reichold
+Copyright 2012-2015, 2018 Adam Reichold
 Copyright 2012 Alexander Volkov
 
 This file is part of qpdfview.
@@ -95,13 +95,16 @@ public:
         void setAnnotationColor(const QColor& annotationColor);
 
         Qt::KeyboardModifiers copyToClipboardModifiers() const;
-        void setCopyToClipboardModifiers(const Qt::KeyboardModifiers& modifiers);
+        void setCopyToClipboardModifiers(Qt::KeyboardModifiers modifiers);
 
         Qt::KeyboardModifiers addAnnotationModifiers() const;
-        void setAddAnnotationModifiers(const Qt::KeyboardModifiers& modifiers);
+        void setAddAnnotationModifiers(Qt::KeyboardModifiers modifiers);
 
         Qt::KeyboardModifiers zoomToSelectionModifiers() const;
-        void setZoomToSelectionModifiers(const Qt::KeyboardModifiers& modifiers);
+        void setZoomToSelectionModifiers(Qt::KeyboardModifiers modifiers);
+
+        Qt::KeyboardModifiers openInSourceEditorModifiers() const;
+        void setOpenInSourceEditorModifiers(Qt::KeyboardModifiers modifiers);
 
         bool annotationOverlay() const;
         void setAnnotationOverlay(bool overlay);
@@ -215,6 +218,9 @@ public:
         bool wholeWords() const;
         void setWholeWords(bool wholeWords);
 
+        bool parallelSearchExecution() const;
+        void setParallelSearchExecution(bool parallelSearchExecution);
+
         int highlightDuration() const;
         void setHighlightDuration(int highlightDuration);
 
@@ -222,13 +228,13 @@ public:
         void setSourceEditor(const QString& sourceEditor);
 
         Qt::KeyboardModifiers zoomModifiers() const;
-        void setZoomModifiers(const Qt::KeyboardModifiers& zoomModifiers);
+        void setZoomModifiers(Qt::KeyboardModifiers zoomModifiers);
 
         Qt::KeyboardModifiers rotateModifiers() const;
-        void setRotateModifiers(const Qt::KeyboardModifiers& rotateModifiers);
+        void setRotateModifiers(Qt::KeyboardModifiers rotateModifiers);
 
         Qt::KeyboardModifiers scrollModifiers() const;
-        void setScrollModifiers(const Qt::KeyboardModifiers& scrollModifiers);
+        void setScrollModifiers(Qt::KeyboardModifiers scrollModifiers);
 
         // per-tab settings
 
@@ -317,9 +323,14 @@ public:
         bool restorePerFileSettings() const;
         void setRestorePerFileSettings(bool restorePerFileSettings);
 
+        int perFileSettingsLimit() const;
+
         int saveDatabaseInterval() const;
         void setSaveDatabaseInterval(int saveDatabaseInterval);
 
+        int currentTabIndex() const;
+        void setCurrentTabIndex(int currentTabIndex);
+
         int tabPosition() const;
         void setTabPosition(int tabPosition);
 
@@ -353,6 +364,9 @@ public:
         bool synchronizeOutlineView() const;
         void setSynchronizeOutlineView(bool synchronizeOutlineView);
 
+        bool synchronizeSplitViews() const;
+        void setSynchronizeSplitViews(bool synchronizeSplitViews);
+
         QStringList fileToolBar() const;
         void setFileToolBar(const QStringList& fileToolBar);
 
@@ -395,14 +409,14 @@ public:
         QString savePath() const;
         void setSavePath(const QString& savePath);
 
-        QSize settingsDialogSize(const QSize& sizeHint) const;
-        void setSettingsDialogSize(const QSize& settingsDialogSize);
+        QSize settingsDialogSize(QSize sizeHint) const;
+        void setSettingsDialogSize(QSize settingsDialogSize);
 
-        QSize fontsDialogSize(const QSize& sizeHint) const;
-        void setFontsDialogSize(const QSize& fontsDialogSize);
+        QSize fontsDialogSize(QSize sizeHint) const;
+        void setFontsDialogSize(QSize fontsDialogSize);
 
-        QSize contentsDialogSize(const QSize& sizeHint) const;
-        void setContentsDialogSize(const QSize& contentsDialogSize);
+        QSize contentsDialogSize(QSize sizeHint) const;
+        void setContentsDialogSize(QSize contentsDialogSize);
 
     private:
         MainWindow(QSettings* settings);
@@ -488,7 +502,7 @@ public:
     class PageItem
     {
     public:
-        static int cacheSize() { return 32 * 1024 * 1024; }
+        static int cacheSize() { return 32 * 1024; }
 
         static bool useTiling() { return false; }
         static int tileSize() { return 1024; }
@@ -509,6 +523,7 @@ public:
         static Qt::KeyboardModifiers copyToClipboardModifiers() { return Qt::ShiftModifier; }
         static Qt::KeyboardModifiers addAnnotationModifiers() { return Qt::ControlModifier; }
         static Qt::KeyboardModifiers zoomToSelectionModifiers() { return Qt::ShiftModifier | Qt::ControlModifier; }
+        static Qt::KeyboardModifiers openInSourceEditorModifiers() { return Qt::NoModifier; }
 
         static bool annotationOverlay() { return false; }
         static bool formFieldOverlay() { return true; }
@@ -566,6 +581,7 @@ public:
 
         static bool matchCase() { return false; }
         static bool wholeWords() { return false; }
+        static bool parallelSearchExecution() { return false; }
 
         static int highlightDuration() { return 5 * 1000; }
         static QString sourceEditor() { return QString(); }
@@ -585,7 +601,7 @@ public:
         static Rotation rotation() { return RotateBy0; }
 
         static bool invertColors() { return false; }
-        static bool convertToGrayscale() { return false; }        
+        static bool convertToGrayscale() { return false; }
         static bool trimMargins() { return false; }
 
         static bool highlightAll() { return false; }
@@ -608,6 +624,8 @@ public:
         static bool restoreBookmarks() { return false; }
         static bool restorePerFileSettings() { return false; }
 
+        static int perFileSettingsLimit() { return 1000; }
+
         static int saveDatabaseInterval() { return 5 * 60 * 1000; }
 
         static int tabPosition() { return 0; }
@@ -628,13 +646,14 @@ public:
         static bool usePageLabel() { return true; }
 
         static bool synchronizeOutlineView() { return false; }
+        static bool synchronizeSplitViews() { return true; }
 
         static QStringList fileToolBar() { return QStringList() << "openInNewTab" << "refresh"; }
         static QStringList editToolBar() { return QStringList() << "currentPage" << "previousPage" << "nextPage"; }
         static QStringList viewToolBar() { return QStringList() << "scaleFactor" << "zoomIn" << "zoomOut"; }
 
         static QStringList documentContextMenu() { return QStringList() << "previousPage" << "nextPage" << "firstPage" << "lastPage" << "separator" << "jumpToPage" << "jumpBackward" << "jumpForward" << "separator" << "setFirstPage" << "separator" << "findPrevious" << "findNext" << "cancelSearch"; }
-        static QStringList tabContexntMenu() { return QStringList() << "openCopyInNewTab" << "openContainingFolder" << "separator" << "closeAllTabs" << "closeAllTabsButThisOne" << "closeAllTabsToTheLeft" << "closeAllTabsToTheRight"; }
+        static QStringList tabContexntMenu() { return QStringList() << "openCopyInNewTab" << "openCopyInNewWindow" << "openContainingFolder" << "separator" << "splitViewHorizontally" << "splitViewVertically" << "closeCurrentView" << "separator" << "closeAllTabs" << "closeAllTabsButThisOne" << "closeAllTabsToTheLeft" << "closeAllTabsToTheRight"; }
 
         static bool scrollableMenus() { return false; }
         static bool searchableMenus() { return false; }
