@@ -26,6 +26,9 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 class QSocketNotifier;
 
+namespace qpdfview
+{
+
 class SignalHandler : public QObject
 {
     Q_OBJECT
@@ -53,4 +56,6 @@ private:
 
 };
 
+} // qpdfview
+
 #endif // SIGNALHANDLER_H
