@@ -34,19 +34,13 @@ ThumbnailItem::ThumbnailItem(Model::Page* page, const QString& text, int index,
     m_isHighlighted(false)
 {
     setAcceptHoverEvents(false);
+
+    prepareToolTip();
 }
 
 QRectF ThumbnailItem::boundingRect() const
 {
-#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
-
-    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, 2.0 * m_text.size().height());
-
-#else
-
-    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, 2.0 * QFontMetrics(QFont()).height());
-
-#endif // QT_VERSION
+    return PageItem::boundingRect().adjusted(0.0, 0.0, 0.0, textHeight());
 }
 
 void ThumbnailItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
@@ -97,6 +91,19 @@ void ThumbnailItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* opt
     }
 }
 
+qreal ThumbnailItem::textHeight() const
+{
+#if QT_VERSION >= QT_VERSION_CHECK(4,7,0)
+
+    return 2.0 * m_text.size().height();
+
+#else
+
+    return 2.0 * QFontMetrics(QFont()).height();
+
+#endif // QT_VERSION
+}
+
 void ThumbnailItem::setHighlighted(bool highlighted)
 {
     if(m_isHighlighted != highlighted)
@@ -137,7 +144,7 @@ void ThumbnailItem::contextMenuEvent(QGraphicsSceneContextMenuEvent*)
 {
 }
 
-void ThumbnailItem::loadInteractiveElements()
+void ThumbnailItem::prepareToolTip()
 {
     const qreal width = size().width() / 72.0 * 25.4;
     const qreal height = size().height() / 72.0 * 25.4;
@@ -159,16 +166,16 @@ void ThumbnailItem::loadInteractiveElements()
         qreal longEdgeB = 1414.0;
         qreal shortEdgeB = 1000.0;
 
-        for(int i = 0; i <= 10; ++i)
+        for(int size = 0; size <= 10; ++size)
         {
             if(qAbs(longEdge - longEdgeA) <= 1.0 && qAbs(shortEdge - shortEdgeA) <= 1.0)
             {
-                paperSize = QString(" (A%1)").arg(i);
+                paperSize = QString(" (A%1)").arg(size);
                 break;
             }
             else if(qAbs(longEdge - longEdgeB) <= 1.0 && qAbs(shortEdge - shortEdgeB) <= 1.0)
             {
-                paperSize = QString(" (B%1)").arg(i);
+                paperSize = QString(" (B%1)").arg(size);
                 break;
             }
 
